@@ -9,7 +9,7 @@ from threading import Thread
 import numpy as np
 import tensorflow as tf
 
-from garage.sampler.utils import rollout as default_rollout
+from garage import rollout as default_rollout
 
 __all__ = ['Plotter']
 
@@ -25,17 +25,17 @@ Message = namedtuple('Message', ['op', 'args', 'kwargs'])
 
 
 class Plotter:
-    """Renders rollouts of the policy as it trains.
+    """Visualizes episodes of the policy as it trains.
 
-    Usually, this class is used by sending plot=True to LocalRunner.train().
+    Usually, this class is used by sending plot=True to Trainer.train().
 
     Args:
-        env (gym.Env): The environment to perform rollouts in. This will be
-        used without copying in the current process but in a separate thread,
-        so it should be given a unique copy (in particular, do not pass the
-        environment here, then try to pickle it, or you will occasionally get
-        crashes).
-        policy (garage.tf.Policy): The policy to do the rollouts with.
+        env (garage.Environment): Environment from which to visualize episodes.
+            This will be used without copying in the current process but in a
+            separate thread, so it should be given a unique copy (in
+            particular, do not pass the training environment here, then try to
+            pickle it, or you will occasionally get crashes).
+        policy (garage.tf.Policy): Policy used to visualize episodes.
         sess (tf.Session): The TensorFlow session to use.
         graph (tf.Graph): The TensorFlow graph to use.
         rollout (callable): The rollout function to call.
@@ -66,9 +66,8 @@ class Plotter:
         if 'Darwin' in platform.platform():
             self.rollout(self._env,
                          self._policy,
-                         max_path_length=np.inf,
-                         animated=True,
-                         speedup=5)
+                         max_episode_length=np.inf,
+                         animated=True)
 
     def _start_worker(self):
         max_length = None
@@ -105,17 +104,15 @@ class Plotter:
                         initial_rollout = False
                         self.rollout(self._env,
                                      self._policy,
-                                     max_path_length=max_length,
-                                     animated=True,
-                                     speedup=5)
+                                     max_episode_length=max_length,
+                                     animated=True)
                         self.queue.task_done()
                     else:
                         if max_length:
                             self.rollout(self._env,
                                          self._policy,
-                                         max_path_length=max_length,
-                                         animated=True,
-                                         speedup=5)
+                                         max_episode_length=max_length,
+                                         animated=True)
         except KeyboardInterrupt:
             pass
 
@@ -154,9 +151,9 @@ class Plotter:
         """Update the policy being plotted.
 
         Args:
-            policy (garage.tf.Policy): The policy to rollout.
-            max_length (int or float): The maximum length to allow a rollout to
-                be. Defaults to infinity.
+            policy (garage.tf.Policy): Policy to visualize.
+            max_length (int or float): The maximum length to allow an episode
+                to be. Defaults to infinity.
 
         """
         if self.worker_thread.is_alive():
