@@ -1,18 +1,17 @@
 """This modules creates a DDPG model in PyTorch."""
-from collections import deque
+# yapf: disable
 import copy
 
 from dowel import logger, tabular
 import numpy as np
 import torch
 
-from garage import _Default, make_optimizer
-from garage import log_performance
-from garage.np import obtain_evaluation_samples
-from garage.np import samples_to_tensors
+from garage import (_Default, log_performance, make_optimizer,
+                    obtain_evaluation_episodes)
 from garage.np.algos import RLAlgorithm
-from garage.sampler import OffPolicyVectorizedSampler
-from garage.torch import dict_np_to_torch, torch_to_np
+from garage.torch import as_torch_dict, torch_to_np
+
+# yapf: enable
 
 
 class DDPG(RLAlgorithm):
@@ -28,20 +27,19 @@ class DDPG(RLAlgorithm):
         env_spec (EnvSpec): Environment specification.
         policy (garage.torch.policies.Policy): Policy.
         qf (object): Q-value network.
-        replay_buffer (garage.replay_buffer.ReplayBuffer): Replay buffer.
+        replay_buffer (ReplayBuffer): Replay buffer.
+        sampler (garage.sampler.Sampler): Sampler.
         steps_per_epoch (int): Number of train_once calls per epoch.
         n_train_steps (int): Training steps.
-        max_path_length (int): Maximum path length. The episode will
-            terminate when length of trajectory reaches max_path_length.
-        max_eval_path_length (int or None): Maximum length of paths used for
-            off-policy evaluation. If None, defaults to `max_path_length`.
         buffer_batch_size (int): Batch size of replay buffer.
         min_buffer_size (int): The minimum buffer size for replay buffer.
-        rollout_batch_size (int): Roll out batch size.
         exploration_policy (garage.np.exploration_policies.ExplorationPolicy): # noqa: E501
                 Exploration strategy.
         target_update_tau (float): Interpolation parameter for doing the
             soft target update.
+        max_episode_length_eval (int or None): Maximum length of episodes used
+            for off-policy evaluation. If `None`, defaults to
+            `env_spec.max_episode_length`.
         discount(float): Discount factor for the cumulative return.
         policy_weight_decay (float): L2 weight decay factor for parameters
             of the policy network.
@@ -64,7 +62,6 @@ class DDPG(RLAlgorithm):
             clip_return].
         max_action (float): Maximum action magnitude.
         reward_scale (float): Reward scale.
-        smooth_return (bool): Whether to smooth the return for logging.
 
     """
 
@@ -74,14 +71,13 @@ class DDPG(RLAlgorithm):
             policy,
             qf,
             replay_buffer,
+            sampler,
             *,  # Everything after this is numbers.
             steps_per_epoch=20,
             n_train_steps=50,
-            max_path_length=None,
-            max_eval_path_length=None,
+            max_episode_length_eval=None,
             buffer_batch_size=64,
             min_buffer_size=int(1e4),
-            rollout_batch_size=1,
             exploration_policy=None,
             target_update_tau=0.01,
             discount=0.99,
@@ -94,8 +90,7 @@ class DDPG(RLAlgorithm):
             clip_pos_returns=False,
             clip_return=np.inf,
             max_action=None,
-            reward_scale=1.,
-            smooth_return=True):
+            reward_scale=1.):
         action_bound = env_spec.action_space.high
         self._tau = target_update_tau
         self._policy_weight_decay = policy_weight_decay
@@ -105,8 +100,6 @@ class DDPG(RLAlgorithm):
         self._max_action = action_bound if max_action is None else max_action
 
         self._steps_per_epoch = steps_per_epoch
-        self._success_history = deque(maxlen=100)
-        self._episode_rewards = []
         self._episode_policy_losses = []
         self._episode_qf_losses = []
         self._epoch_ys = []
@@ -123,13 +116,13 @@ class DDPG(RLAlgorithm):
         self._buffer_batch_size = buffer_batch_size
         self._discount = discount
         self._reward_scale = reward_scale
-        self._smooth_return = smooth_return
-        self.max_path_length = max_path_length
-        self._max_eval_path_length = max_eval_path_length
+        self.max_episode_length = env_spec.max_episode_length
+        self._max_episode_length_eval = env_spec.max_episode_length
+
+        if max_episode_length_eval is not None:
+            self._max_episode_length_eval = max_episode_length_eval
 
-        # used by OffPolicyVectorizedSampler
         self.env_spec = env_spec
-        self.rollout_batch_size = rollout_batch_size
         self.replay_buffer = replay_buffer
         self.policy = policy
         self.exploration_policy = exploration_policy
@@ -142,88 +135,61 @@ class DDPG(RLAlgorithm):
         self._qf_optimizer = make_optimizer(qf_optimizer,
                                             module=self._qf,
                                             lr=qf_lr)
+        self._eval_env = None
+        self._sampler = sampler
 
-        self.sampler_cls = OffPolicyVectorizedSampler
-
-    def train(self, runner):
+    def train(self, trainer):
         """Obtain samplers and start actual training for each epoch.
 
         Args:
-            runner (LocalRunner): LocalRunner is passed to give algorithm
-                the access to runner.step_epochs(), which provides services
-                such as snapshotting and sampler control.
+            trainer (Trainer): Experiment trainer.
 
         Returns:
             float: The average return in last epoch cycle.
 
         """
-        last_return = None
-        runner.enable_logging = False
+        if not self._eval_env:
+            self._eval_env = trainer.get_env_copy()
+        last_returns = [float('nan')]
+        trainer.enable_logging = False
 
-        for _ in runner.step_epochs():
+        for _ in trainer.step_epochs():
             for cycle in range(self._steps_per_epoch):
-                runner.step_path = runner.obtain_samples(runner.step_itr)
+                trainer.step_episode = trainer.obtain_episodes(
+                    trainer.step_itr)
                 if hasattr(self.exploration_policy, 'update'):
-                    self.exploration_policy.update(runner.step_path)
-                for path in runner.step_path:
-                    path['rewards'] *= self._reward_scale
-                last_return = self.train_once(runner.step_itr,
-                                              runner.step_path)
+                    self.exploration_policy.update(trainer.step_episode)
+                self.train_once(trainer.step_itr, trainer.step_episode)
                 if (cycle == 0 and self.replay_buffer.n_transitions_stored >=
                         self._min_buffer_size):
-                    runner.enable_logging = True
-                    log_performance(runner.step_itr,
-                                    obtain_evaluation_samples(
-                                        self.policy, runner.get_env_copy()),
-                                    discount=self._discount)
-                runner.step_itr += 1
+                    trainer.enable_logging = True
+                    eval_eps = obtain_evaluation_episodes(
+                        self.policy, self._eval_env)
+                    last_returns = log_performance(trainer.step_itr,
+                                                   eval_eps,
+                                                   discount=self._discount)
+                trainer.step_itr += 1
 
-        return last_return
+        return np.mean(last_returns)
 
-    def train_once(self, itr, paths):
+    def train_once(self, itr, episodes):
         """Perform one iteration of training.
 
         Args:
             itr (int): Iteration number.
-            paths (list[dict]): A list of collected paths
-
-        Returns:
-            float: Average return.
+            episodes (EpisodeBatch): Batch of episodes.
 
         """
-        paths = samples_to_tensors(paths)
+        self.replay_buffer.add_episode_batch(episodes)
 
         epoch = itr / self._steps_per_epoch
 
-        self._episode_rewards.extend([
-            path for path, complete in zip(paths['undiscounted_returns'],
-                                           paths['complete']) if complete
-        ])
-        self._success_history.extend([
-            path for path, complete in zip(paths['success_history'],
-                                           paths['complete']) if complete
-        ])
-
-        # Avoid calculating the mean of an empty list in cases where
-        # all paths were non-terminal.
-
-        last_average_return = np.NaN
-        avg_success_rate = 0
-
-        if self._episode_rewards:
-            last_average_return = np.mean(self._episode_rewards)
-
-        if self._success_history:
-            if (itr % self._steps_per_epoch == 0
-                    and (self.replay_buffer.n_transitions_stored >=
-                         self._min_buffer_size)):
-                avg_success_rate = np.mean(self._success_history)
-
         for _ in range(self._n_train_steps):
             if (self.replay_buffer.n_transitions_stored >=
                     self._min_buffer_size):
                 samples = self.replay_buffer.sample_transitions(
                     self._buffer_batch_size)
+                samples['rewards'] *= self._reward_scale
                 qf_loss, y, q, policy_loss = torch_to_np(
                     self.optimize_policy(samples))
 
@@ -250,18 +216,6 @@ class DDPG(RLAlgorithm):
                 tabular.record('QFunction/MaxY', np.max(self._epoch_ys))
                 tabular.record('QFunction/AverageAbsY',
                                np.mean(np.abs(self._epoch_ys)))
-                tabular.record('AverageSuccessRate', avg_success_rate)
-
-            if not self._smooth_return:
-                self._episode_rewards = []
-                self._episode_policy_losses = []
-                self._episode_qf_losses = []
-                self._epoch_ys = []
-                self._epoch_qs = []
-
-            self._success_history.clear()
-
-        return last_average_return
 
     def optimize_policy(self, samples_data):
         """Perform algorithm optimizing.
@@ -276,7 +230,7 @@ class DDPG(RLAlgorithm):
             qval: Q-value predicted by the Q-network.
 
         """
-        transitions = dict_np_to_torch(samples_data)
+        transitions = as_torch_dict(samples_data)
 
         observations = transitions['observations']
         rewards = transitions['rewards'].reshape(-1, 1)
