@@ -1,4 +1,5 @@
 """Model-Agnostic Meta-Learning (MAML) algorithm implementation for RL."""
+# yapf: disable
 import collections
 import copy
 
@@ -6,14 +7,14 @@ from dowel import tabular
 import numpy as np
 import torch
 
-from garage import _Default, make_optimizer
-from garage import log_multitask_performance
-from garage import TrajectoryBatch
-from garage.misc import tensor_utils
-from garage.sampler import OnPolicyVectorizedSampler
+from garage import (_Default, EpisodeBatch, log_multitask_performance,
+                    make_optimizer)
+from garage.np import discount_cumsum
 from garage.torch import update_module_params
-from garage.torch.optimizers import ConjugateGradientOptimizer
-from garage.torch.optimizers import DifferentiableSGD
+from garage.torch.optimizers import (ConjugateGradientOptimizer,
+                                     DifferentiableSGD)
+
+# yapf: enable
 
 
 class MAML:
@@ -22,8 +23,10 @@ class MAML:
     Args:
         inner_algo (garage.torch.algos.VPG): The inner algorithm used for
             computing loss.
-        env (garage.envs.GarageEnv): A gym environment.
+        env (Environment): An environment.
         policy (garage.torch.policies.Policy): Policy.
+        sampler (garage.sampler.Sampler): Sampler.
+        task_sampler (garage.experiment.TaskSampler): Task sampler.
         meta_optimizer (Union[torch.optim.Optimizer, tuple]):
             Type of optimizer.
             This can be an optimizer type such as `torch.optim.Adam` or a tuple
@@ -33,8 +36,8 @@ class MAML:
         inner_lr (float): Adaptation learning rate.
         outer_lr (float): Meta policy learning rate.
         num_grad_updates (int): Number of adaptation gradient steps.
-        meta_evaluator (garage.experiment.MetaEvaluator): A meta evaluator for
-            meta-testing. If None, don't do meta-testing.
+        meta_evaluator (MetaEvaluator): A meta evaluator for meta-testing. If
+            None, don't do meta-testing.
         evaluate_every_n_epochs (int): Do meta-testing every this epochs.
 
     """
@@ -43,6 +46,8 @@ class MAML:
                  inner_algo,
                  env,
                  policy,
+                 sampler,
+                 task_sampler,
                  meta_optimizer,
                  meta_batch_size=40,
                  inner_lr=0.1,
@@ -50,13 +55,14 @@ class MAML:
                  num_grad_updates=1,
                  meta_evaluator=None,
                  evaluate_every_n_epochs=1):
-        self.sampler_cls = OnPolicyVectorizedSampler
+        self._sampler = sampler
 
-        self.max_path_length = inner_algo.max_path_length
+        self.max_episode_length = inner_algo.max_episode_length
 
         self._meta_evaluator = meta_evaluator
         self._policy = policy
         self._env = env
+        self._task_sampler = task_sampler
         self._value_function = copy.deepcopy(inner_algo._value_function)
         self._initial_vf_state = self._value_function.state_dict()
         self._num_grad_updates = num_grad_updates
@@ -69,12 +75,12 @@ class MAML:
                                               eps=_Default(1e-5))
         self._evaluate_every_n_epochs = evaluate_every_n_epochs
 
-    def train(self, runner):
+    def train(self, trainer):
         """Obtain samples and start training for each epoch.
 
         Args:
-            runner (LocalRunner): LocalRunner is passed to give algorithm
-                the access to runner.step_epochs(), which provides services
+            trainer (Trainer): Gives the algorithm access to
+                :method:`~Trainer.step_epochs()`, which provides services
                 such as snapshotting and sampler control.
 
         Returns:
@@ -83,20 +89,20 @@ class MAML:
         """
         last_return = None
 
-        for _ in runner.step_epochs():
-            all_samples, all_params = self._obtain_samples(runner)
-            last_return = self.train_once(runner, all_samples, all_params)
-            runner.step_itr += 1
+        for _ in trainer.step_epochs():
+            all_samples, all_params = self._obtain_samples(trainer)
+            last_return = self._train_once(trainer, all_samples, all_params)
+            trainer.step_itr += 1
 
         return last_return
 
-    def train_once(self, runner, all_samples, all_params):
+    def _train_once(self, trainer, all_samples, all_params):
         """Train the algorithm once.
 
         Args:
-            runner (garage.experiment.LocalRunner): The experiment runner.
-            all_samples (list[list[MAMLTrajectoryBatch]]): A two
-                dimensional list of MAMLTrajectoryBatch of size
+            trainer (Trainer): The experiment runner.
+            all_samples (list[list[_MAMLEpisodeBatch]]): A two
+                dimensional list of _MAMLEpisodeBatch of size
                 [meta_batch_size * (num_grad_updates + 1)]
             all_params (list[dict]): A list of named parameter dictionaries.
                 Each dictionary contains key value pair of names (str) and
@@ -106,7 +112,7 @@ class MAML:
             float: Average return.
 
         """
-        itr = runner.step_itr
+        itr = trainer.step_itr
         old_theta = dict(self._policy.named_parameters())
 
         kl_before = self._compute_kl_constraint(all_samples,
@@ -131,12 +137,10 @@ class MAML:
         with torch.no_grad():
             policy_entropy = self._compute_policy_entropy(
                 [task_samples[0] for task_samples in all_samples])
-            average_return = self.log_performance(itr, all_samples,
-                                                  meta_objective.item(),
-                                                  loss_after.item(),
-                                                  kl_before.item(),
-                                                  kl_after.item(),
-                                                  policy_entropy.mean().item())
+            average_return = self._log_performance(
+                itr, all_samples, meta_objective.item(), loss_after.item(),
+                kl_before.item(), kl_after.item(),
+                policy_entropy.mean().item())
 
         if self._meta_evaluator and itr % self._evaluate_every_n_epochs == 0:
             self._meta_evaluator.evaluate(self)
@@ -173,31 +177,31 @@ class MAML:
 
         return vf_loss
 
-    def _obtain_samples(self, runner):
+    def _obtain_samples(self, trainer):
         """Obtain samples for each task before and after the fast-adaptation.
 
         Args:
-            runner (LocalRunner): A local runner instance to obtain samples.
+            trainer (Trainer): A trainer instance to obtain samples.
 
         Returns:
             tuple: Tuple of (all_samples, all_params).
-                all_samples (list[MAMLTrajectoryBatch]): A list of size
+                all_samples (list[_MAMLEpisodeBatch]): A list of size
                     [meta_batch_size * (num_grad_updates + 1)]
                 all_params (list[dict]): A list of named parameter
                     dictionaries.
 
         """
-        tasks = self._env.sample_tasks(self._meta_batch_size)
+        tasks = self._task_sampler.sample(self._meta_batch_size)
         all_samples = [[] for _ in range(len(tasks))]
         all_params = []
         theta = dict(self._policy.named_parameters())
 
-        for i, task in enumerate(tasks):
-            self._set_task(runner, task)
+        for i, env_up in enumerate(tasks):
 
             for j in range(self._num_grad_updates + 1):
-                paths = runner.obtain_samples(runner.step_itr)
-                batch_samples = self._process_samples(paths)
+                episodes = trainer.obtain_episodes(trainer.step_itr,
+                                                   env_update=env_up)
+                batch_samples = self._process_samples(episodes)
                 all_samples[i].append(batch_samples)
 
                 # The last iteration does only sampling but no adapting
@@ -217,7 +221,7 @@ class MAML:
         """Performs one MAML inner step to update the policy.
 
         Args:
-            batch_samples (MAMLTrajectoryBatch): Samples data for one
+            batch_samples (_MAMLEpisodeBatch): Samples data for one
                 task and one gradient step.
             set_grad (bool): if False, update policy parameters in-place.
                 Else, allow taking gradient of functions of updated parameters
@@ -249,8 +253,8 @@ class MAML:
         """Compute loss to meta-optimize.
 
         Args:
-            all_samples (list[list[MAMLTrajectoryBatch]]): A two
-                dimensional list of MAMLTrajectoryBatch of size
+            all_samples (list[list[_MAMLEpisodeBatch]]): A two
+                dimensional list of _MAMLEpisodeBatch of size
                 [meta_batch_size * (num_grad_updates + 1)]
             all_params (list[dict]): A list of named parameter dictionaries.
                 Each dictionary contains key value pair of names (str) and
@@ -289,8 +293,8 @@ class MAML:
         distribution and current policy distribution.
 
         Args:
-            all_samples (list[list[MAMLTrajectoryBatch]]): Two
-                dimensional list of MAMLTrajectoryBatch of size
+            all_samples (list[list[_MAMLEpisodeBatch]]): Two
+                dimensional list of _MAMLEpisodeBatch of size
                 [meta_batch_size * (num_grad_updates + 1)]
             all_params (list[dict]): A list of named parameter dictionaries.
                 Each dictionary contains key value pair of names (str) and
@@ -326,23 +330,18 @@ class MAML:
         """Compute policy entropy.
 
         Args:
-            task_samples (list[MAMLTrajectoryBatch]): Samples data for
+            task_samples (list[_MAMLEpisodeBatch]): Samples data for
                 one task.
 
         Returns:
             torch.Tensor: Computed entropy value.
 
         """
-        obs = torch.stack([samples.observations for samples in task_samples])
+        obs = torch.cat([samples.observations for samples in task_samples])
         # pylint: disable=protected-access
         entropies = self._inner_algo._compute_policy_entropy(obs)
         return entropies.mean()
 
-    def _set_task(self, runner, task):
-        # pylint: disable=protected-access, no-self-use
-        for env in runner._sampler._vec_env.envs:
-            env.set_task(task)
-
     @property
     def policy(self):
         """Current policy of the inner algorithm.
@@ -365,34 +364,42 @@ class MAML:
         # pylint: disable=protected-access
         return self._inner_algo._old_policy
 
-    def _process_samples(self, paths):
+    def _process_samples(self, episodes):
         """Process sample data based on the collected paths.
 
         Args:
-            paths (list[dict]): A list of collected paths.
+            episodes (EpisodeBatch): Collected batch of episodes.
 
         Returns:
-            MAMLTrajectoryBatch: Processed samples data.
+            _MAMLEpisodeBatch: Processed samples data.
 
         """
+        paths = episodes.to_list()
         for path in paths:
-            path['returns'] = tensor_utils.discount_cumsum(
+            path['returns'] = discount_cumsum(
                 path['rewards'], self._inner_algo.discount).copy()
 
         self._train_value_function(paths)
-        obs, actions, rewards, _, valids, baselines \
-            = self._inner_algo.process_samples(paths)
-        return MAMLTrajectoryBatch(paths, obs, actions, rewards, valids,
-                                   baselines)
 
-    def log_performance(self, itr, all_samples, loss_before, loss_after,
-                        kl_before, kl, policy_entropy):
+        obs = torch.Tensor(episodes.padded_observations)
+        actions = torch.Tensor(episodes.padded_actions)
+        rewards = torch.Tensor(episodes.padded_rewards)
+        valids = torch.Tensor(episodes.lengths).int()
+        with torch.no_grad():
+            # pylint: disable=protected-access
+            baselines = self._inner_algo._value_function(obs)
+
+        return _MAMLEpisodeBatch(paths, obs, actions, rewards, valids,
+                                 baselines)
+
+    def _log_performance(self, itr, all_samples, loss_before, loss_after,
+                         kl_before, kl, policy_entropy):
         """Evaluate performance of this batch.
 
         Args:
             itr (int): Iteration number.
-            all_samples (list[list[MAMLTrajectoryBatch]]): Two
-                dimensional list of MAMLTrajectoryBatch of size
+            all_samples (list[list[_MAMLEpisodeBatch]]): Two
+                dimensional list of _MAMLEpisodeBatch of size
                 [meta_batch_size * (num_grad_updates + 1)]
             loss_before (float): Loss before optimization step.
             loss_after (float): Loss after optimization step.
@@ -413,7 +420,7 @@ class MAML:
 
         rtns = log_multitask_performance(
             itr,
-            TrajectoryBatch.from_trajectory_list(
+            EpisodeBatch.from_list(
                 env_spec=self._env.spec,
                 paths=[
                     path for task_paths in all_samples
@@ -439,36 +446,34 @@ class MAML:
         task.
 
         Returns:
-            garage.Policy: The policy used to obtain samples that are later
-                used for meta-RL adaptation.
+            Policy: The policy used to obtain samples that are later used for
+                meta-RL adaptation.
 
         """
         return copy.deepcopy(self._policy)
 
-    def adapt_policy(self, exploration_policy, exploration_trajectories):
+    def adapt_policy(self, exploration_policy, exploration_episodes):
         """Adapt the policy by one gradient steps for a task.
 
         Args:
-            exploration_policy (garage.Policy): A policy which was returned
-                from get_exploration_policy(), and which generated
-                exploration_trajectories by interacting with an environment.
+            exploration_policy (Policy): A policy which was returned from
+                get_exploration_policy(), and which generated
+                exploration_episodes by interacting with an environment.
                 The caller may not use this object after passing it into this
                 method.
-            exploration_trajectories (garage.TrajectoryBatch): Trajectories to
-                adapt to, generated by exploration_policy exploring the
-                environment.
+            exploration_episodes (EpisodeBatch): Episodes with which to adapt,
+                generated by exploration_policy exploring the environment.
 
         Returns:
-            garage.Policy: A policy adapted to the task represented by the
-                exploration_trajectories.
+            Policy: A policy adapted to the task represented by the
+                exploration_episodes.
 
         """
         old_policy, self._policy = self._policy, exploration_policy
         self._inner_algo.policy = exploration_policy
         self._inner_optimizer.module = exploration_policy
 
-        paths = exploration_trajectories.to_trajectory_list()
-        batch_samples = self._process_samples(paths)
+        batch_samples = self._process_samples(exploration_episodes)
 
         self._adapt(batch_samples, set_grad=False)
 
@@ -477,21 +482,21 @@ class MAML:
         return exploration_policy
 
 
-class MAMLTrajectoryBatch(
-        collections.namedtuple('MAMLTrajectoryBatch', [
+class _MAMLEpisodeBatch(
+        collections.namedtuple('_MAMLEpisodeBatch', [
             'paths', 'observations', 'actions', 'rewards', 'valids',
             'baselines'
         ])):
-    r"""A tuple representing a batch of whole trajectories in MAML.
+    r"""A tuple representing a batch of whole episodes in MAML.
 
-    A :class:`MAMLTrajectoryBatch` represents a batch of whole trajectories
+    A :class:`_MAMLEpisodeBatch` represents a batch of whole episodes
     produced from one environment.
     +-----------------------+-------------------------------------------------+
     | Symbol                | Description                                     |
     +=======================+=================================================+
-    | :math:`N`             | Trajectory index dimension                      |
+    | :math:`N`             | Episode batch dimension                         |
     +-----------------------+-------------------------------------------------+
-    | :math:`T`             | Maximum length of a trajectory                  |
+    | :math:`T`             | Maximum length of an episode                    |
     +-----------------------+-------------------------------------------------+
     | :math:`S^*`           | Single-step shape of a time-series tensor       |
     +-----------------------+-------------------------------------------------+
@@ -511,8 +516,8 @@ class MAMLTrajectoryBatch(
             :math:`(N \bullet T)` containing the rewards for all time
             steps in this batch.
         valids (numpy.ndarray): An integer numpy array of shape :math:`(N, )`
-            containing the length of each trajectory in this batch. This may be
-            used to reconstruct the individual trajectories.
+            containing the length of each episode in this batch. This may be
+            used to reconstruct the individual episodes.
         baselines (numpy.ndarray): An numpy array of shape
             :math:`(N \bullet T, )` containing the value function estimation
             at all time steps in this batch.
