@@ -10,8 +10,7 @@ from dowel import logger
 import numpy as np
 import torch
 
-from garage import InOutSpec, TimeStep
-from garage.envs import EnvSpec
+from garage import EnvSpec, InOutSpec, StepType, TimeStep
 from garage.experiment import MetaEvaluator
 from garage.np.algos import MetaRLAlgorithm
 from garage.replay_buffer import PathBuffer
@@ -32,18 +31,19 @@ class PEARL(MetaRLAlgorithm):
     behavior to specific tasks.
 
     Args:
-        env (list[GarageEnv]): Batch of sampled environment updates(EnvUpdate),
-            which, when invoked on environments, will configure them with new
-            tasks.
-        policy_class (garage.torch.policies.Policy): Context-conditioned policy
-            class.
+        env (list[Environment]): Batch of sampled environment updates(
+            EnvUpdate), which, when invoked on environments, will configure
+            them with new tasks.
+        policy_class (type): Class implementing
+            :pyclass:`~ContextConditionedPolicy`
         encoder_class (garage.torch.embeddings.ContextEncoder): Encoder class
             for the encoder in context-conditioned policy.
         inner_policy (garage.torch.policies.Policy): Policy.
         qf (torch.nn.Module): Q-function.
         vf (torch.nn.Module): Value function.
+        sampler (garage.sampler.Sampler): Sampler.
         num_train_tasks (int): Number of tasks for training.
-        num_test_tasks (int): Number of tasks for testing.
+        num_test_tasks (int or None): Number of tasks for testing.
         latent_dim (int): Size of latent context vector.
         encoder_hidden_sizes (list[int]): Output dimension of dense layer(s) of
             the context encoder.
@@ -59,7 +59,7 @@ class PEARL(MetaRLAlgorithm):
         soft_target_tau (float): Interpolation parameter for doing the
             soft target update.
         kl_lambda (float): KL lambda value.
-        optimizer_class (callable): Type of optimizer for training networks.
+        optimizer_class (type): Type of optimizer for training networks.
         use_information_bottleneck (bool): False means latent context is
             deterministic.
         use_next_obs_in_context (bool): Whether or not to use next observation
@@ -82,62 +82,62 @@ class PEARL(MetaRLAlgorithm):
         embedding_mini_batch_size (int): Number of transitions in mini context
             batch; should be same as embedding_batch_size for non-recurrent
             encoder.
-        max_path_length (int): Maximum path length.
         discount (float): RL discount factor.
         replay_buffer_size (int): Maximum samples in replay buffer.
         reward_scale (int): Reward scale.
         update_post_train (int): How often to resample context when obtaining
-            data during training (in trajectories).
+            data during training (in episodes).
 
     """
 
     # pylint: disable=too-many-statements
-    def __init__(self,
-                 env,
-                 inner_policy,
-                 qf,
-                 vf,
-                 num_train_tasks,
-                 num_test_tasks,
-                 latent_dim,
-                 encoder_hidden_sizes,
-                 test_env_sampler,
-                 policy_class=ContextConditionedPolicy,
-                 encoder_class=MLPEncoder,
-                 policy_lr=3E-4,
-                 qf_lr=3E-4,
-                 vf_lr=3E-4,
-                 context_lr=3E-4,
-                 policy_mean_reg_coeff=1E-3,
-                 policy_std_reg_coeff=1E-3,
-                 policy_pre_activation_coeff=0.,
-                 soft_target_tau=0.005,
-                 kl_lambda=.1,
-                 optimizer_class=torch.optim.Adam,
-                 use_information_bottleneck=True,
-                 use_next_obs_in_context=False,
-                 meta_batch_size=64,
-                 num_steps_per_epoch=1000,
-                 num_initial_steps=100,
-                 num_tasks_sample=100,
-                 num_steps_prior=100,
-                 num_steps_posterior=0,
-                 num_extra_rl_steps_posterior=100,
-                 batch_size=1024,
-                 embedding_batch_size=1024,
-                 embedding_mini_batch_size=1024,
-                 max_path_length=1000,
-                 discount=0.99,
-                 replay_buffer_size=1000000,
-                 reward_scale=1,
-                 update_post_train=1):
+    def __init__(
+            self,
+            env,
+            inner_policy,
+            qf,
+            vf,
+            sampler,
+            *,  # Mostly numbers after here.
+            num_train_tasks,
+            num_test_tasks=None,
+            latent_dim,
+            encoder_hidden_sizes,
+            test_env_sampler,
+            policy_class=ContextConditionedPolicy,
+            encoder_class=MLPEncoder,
+            policy_lr=3E-4,
+            qf_lr=3E-4,
+            vf_lr=3E-4,
+            context_lr=3E-4,
+            policy_mean_reg_coeff=1E-3,
+            policy_std_reg_coeff=1E-3,
+            policy_pre_activation_coeff=0.,
+            soft_target_tau=0.005,
+            kl_lambda=.1,
+            optimizer_class=torch.optim.Adam,
+            use_information_bottleneck=True,
+            use_next_obs_in_context=False,
+            meta_batch_size=64,
+            num_steps_per_epoch=1000,
+            num_initial_steps=100,
+            num_tasks_sample=100,
+            num_steps_prior=100,
+            num_steps_posterior=0,
+            num_extra_rl_steps_posterior=100,
+            batch_size=1024,
+            embedding_batch_size=1024,
+            embedding_mini_batch_size=1024,
+            discount=0.99,
+            replay_buffer_size=1000000,
+            reward_scale=1,
+            update_post_train=1):
 
         self._env = env
         self._qf1 = qf
         self._qf2 = copy.deepcopy(qf)
         self._vf = vf
         self._num_train_tasks = num_train_tasks
-        self._num_test_tasks = num_test_tasks
         self._latent_dim = latent_dim
 
         self._policy_mean_reg_coeff = policy_mean_reg_coeff
@@ -158,23 +158,32 @@ class PEARL(MetaRLAlgorithm):
         self._batch_size = batch_size
         self._embedding_batch_size = embedding_batch_size
         self._embedding_mini_batch_size = embedding_mini_batch_size
-        self.max_path_length = max_path_length
         self._discount = discount
         self._replay_buffer_size = replay_buffer_size
         self._reward_scale = reward_scale
         self._update_post_train = update_post_train
         self._task_idx = None
+        self._single_env = env[0]()
+        self.max_episode_length = self._single_env.spec.max_episode_length
+
+        self._sampler = sampler
 
         self._is_resuming = False
 
+        if num_test_tasks is None:
+            num_test_tasks = test_env_sampler.n_tasks
+        if num_test_tasks is None:
+            raise ValueError('num_test_tasks must be provided if '
+                             'test_env_sampler.n_tasks is None')
+
         worker_args = dict(deterministic=True, accum_context=True)
         self._evaluator = MetaEvaluator(test_task_sampler=test_env_sampler,
-                                        max_path_length=max_path_length,
                                         worker_class=PEARLWorker,
                                         worker_args=worker_args,
                                         n_test_tasks=num_test_tasks)
 
-        encoder_spec = self.get_env_spec(env[0](), latent_dim, 'encoder')
+        encoder_spec = self.get_env_spec(self._single_env, latent_dim,
+                                         'encoder')
         encoder_in_dim = int(np.prod(encoder_spec.input_space.shape))
         encoder_out_dim = int(np.prod(encoder_spec.output_space.shape))
         context_encoder = encoder_class(input_dim=encoder_in_dim,
@@ -254,23 +263,23 @@ class PEARL(MetaRLAlgorithm):
         }
         self._is_resuming = True
 
-    def train(self, runner):
+    def train(self, trainer):
         """Obtain samples, train, and evaluate for each epoch.
 
         Args:
-            runner (LocalRunner): LocalRunner is passed to give algorithm
-                the access to runner.step_epochs(), which provides services
+            trainer (Trainer): Gives the algorithm the access to
+                :method:`Trainer..step_epochs()`, which provides services
                 such as snapshotting and sampler control.
 
         """
-        for _ in runner.step_epochs():
-            epoch = runner.step_itr / self._num_steps_per_epoch
+        for _ in trainer.step_epochs():
+            epoch = trainer.step_itr / self._num_steps_per_epoch
 
             # obtain initial set of samples from all train tasks
             if epoch == 0 or self._is_resuming:
                 for idx in range(self._num_train_tasks):
                     self._task_idx = idx
-                    self._obtain_samples(runner, epoch,
+                    self._obtain_samples(trainer, epoch,
                                          self._num_initial_steps, np.inf)
                     self._is_resuming = False
 
@@ -281,16 +290,16 @@ class PEARL(MetaRLAlgorithm):
                 self._context_replay_buffers[idx].clear()
                 # obtain samples with z ~ prior
                 if self._num_steps_prior > 0:
-                    self._obtain_samples(runner, epoch, self._num_steps_prior,
+                    self._obtain_samples(trainer, epoch, self._num_steps_prior,
                                          np.inf)
                 # obtain samples with z ~ posterior
                 if self._num_steps_posterior > 0:
-                    self._obtain_samples(runner, epoch,
+                    self._obtain_samples(trainer, epoch,
                                          self._num_steps_posterior,
                                          self._update_post_train)
                 # obtain extras samples for RL training but not encoder
                 if self._num_extra_rl_steps_posterior > 0:
-                    self._obtain_samples(runner,
+                    self._obtain_samples(trainer,
                                          epoch,
                                          self._num_extra_rl_steps_posterior,
                                          self._update_post_train,
@@ -299,7 +308,7 @@ class PEARL(MetaRLAlgorithm):
             logger.log('Training...')
             # sample train tasks and optimize networks
             self._train_once()
-            runner.step_itr += 1
+            trainer.step_itr += 1
 
             logger.log('Evaluating...')
             # evaluate
@@ -398,7 +407,7 @@ class PEARL(MetaRLAlgorithm):
         self._policy_optimizer.step()
 
     def _obtain_samples(self,
-                        runner,
+                        trainer,
                         itr,
                         num_samples,
                         update_posterior_rate,
@@ -406,10 +415,10 @@ class PEARL(MetaRLAlgorithm):
         """Obtain samples.
 
         Args:
-            runner (LocalRunner): LocalRunner.
+            trainer (Trainer): Trainer.
             itr (int): Index of iteration (epoch).
             num_samples (int): Number of samples to obtain.
-            update_posterior_rate (int): How often (in trajectories) to infer
+            update_posterior_rate (int): How often (in episodes) to infer
                 posterior of policy.
             add_to_enc_buffer (bool): Whether or not to add samples to encoder
                 buffer.
@@ -420,23 +429,31 @@ class PEARL(MetaRLAlgorithm):
 
         if update_posterior_rate != np.inf:
             num_samples_per_batch = (update_posterior_rate *
-                                     self.max_path_length)
+                                     self.max_episode_length)
         else:
             num_samples_per_batch = num_samples
 
         while total_samples < num_samples:
-            paths = runner.obtain_samples(itr, num_samples_per_batch,
-                                          self._policy,
-                                          self._env[self._task_idx])
+            paths = trainer.obtain_samples(itr, num_samples_per_batch,
+                                           self._policy,
+                                           self._env[self._task_idx])
             total_samples += sum([len(path['rewards']) for path in paths])
 
             for path in paths:
                 p = {
-                    'observations': path['observations'],
-                    'actions': path['actions'],
-                    'rewards': path['rewards'].reshape(-1, 1),
-                    'next_observations': path['next_observations'],
-                    'dones': path['dones'].reshape(-1, 1)
+                    'observations':
+                    path['observations'],
+                    'actions':
+                    path['actions'],
+                    'rewards':
+                    path['rewards'].reshape(-1, 1),
+                    'next_observations':
+                    path['next_observations'],
+                    'dones':
+                    np.array([
+                        step_type == StepType.TERMINAL
+                        for step_type in path['step_types']
+                    ]).reshape(-1, 1)
                 }
                 self._replay_buffers[self._task_idx].add_path(p)
 
@@ -568,34 +585,34 @@ class PEARL(MetaRLAlgorithm):
         task.
 
         Returns:
-            garage.Policy: The policy used to obtain samples that are later
-                used for meta-RL adaptation.
+            Policy: The policy used to obtain samples that are later used for
+                meta-RL adaptation.
 
         """
         return self._policy
 
-    def adapt_policy(self, exploration_policy, exploration_trajectories):
+    def adapt_policy(self, exploration_policy, exploration_episodes):
         """Produce a policy adapted for a task.
 
         Args:
-            exploration_policy (garage.Policy): A policy which was returned
-                from get_exploration_policy(), and which generated
-                exploration_trajectories by interacting with an environment.
+            exploration_policy (Policy): A policy which was returned from
+                get_exploration_policy(), and which generated
+                exploration_episodes by interacting with an environment.
                 The caller may not use this object after passing it into this
                 method.
-            exploration_trajectories (garage.TrajectoryBatch): Trajectories to
-                adapt to, generated by exploration_policy exploring the
+            exploration_episodes (EpisodeBatch): Episodes to which to adapt,
+                generated by exploration_policy exploring the
                 environment.
 
         Returns:
-            garage.Policy: A policy adapted to the task represented by the
-                exploration_trajectories.
+            Policy: A policy adapted to the task represented by the
+                exploration_episodes.
 
         """
-        total_steps = sum(exploration_trajectories.lengths)
-        o = exploration_trajectories.observations
-        a = exploration_trajectories.actions
-        r = exploration_trajectories.rewards.reshape(total_steps, 1)
+        total_steps = sum(exploration_episodes.lengths)
+        o = exploration_episodes.observations
+        a = exploration_episodes.actions
+        r = exploration_episodes.rewards.reshape(total_steps, 1)
         ctxt = np.hstack((o, a, r)).reshape(1, total_steps, -1)
         context = torch.as_tensor(ctxt, device=global_device()).float()
         self._policy.infer_posterior(context)
@@ -618,11 +635,11 @@ class PEARL(MetaRLAlgorithm):
         """Augment environment by a size of latent dimension.
 
         Args:
-            env_spec (garage.envs.EnvSpec): Environment specs to be augmented.
+            env_spec (EnvSpec): Environment specs to be augmented.
             latent_dim (int): Latent dimension.
 
         Returns:
-            garage.envs.EnvSpec: Augmented environment specs.
+            EnvSpec: Augmented environment specs.
 
         """
         obs_dim = int(np.prod(env_spec.observation_space.shape))
@@ -642,13 +659,12 @@ class PEARL(MetaRLAlgorithm):
         """Get environment specs of encoder with latent dimension.
 
         Args:
-            env_spec (garage.envs.EnvSpec): Environment specs.
+            env_spec (EnvSpec): Environment specification.
             latent_dim (int): Latent dimension.
             module (str): Module to get environment specs for.
 
         Returns:
-            garage.envs.InOutSpec: Module environment specs with latent
-                dimension.
+            InOutSpec: Module environment specs with latent dimension.
 
         """
         obs_dim = int(np.prod(env_spec.observation_space.shape))
@@ -678,91 +694,84 @@ class PEARLWorker(DefaultWorker):
     It stores context and resample belief in the policy every step.
 
     Args:
-        seed(int): The seed to use to intialize random number generators.
-        max_path_length(int or float): The maximum length paths which will
-            be sampled. Can be (floating point) infinity.
-        worker_number(int): The number of the worker where this update is
+        seed (int): The seed to use to intialize random number generators.
+        max_episode_length(int or float): The maximum length of episodes which
+            will be sampled. Can be (floating point) infinity.
+        worker_number (int): The number of the worker where this update is
             occurring. This argument is used to set a different seed for each
             worker.
-        deterministic(bool): If true, use the mean action returned by the
+        deterministic (bool): If True, use the mean action returned by the
             stochastic policy instead of sampling from the returned action
             distribution.
-        accum_context(bool): If true, update context of the agent.
+        accum_context (bool): If True, update context of the agent.
 
     Attributes:
-        agent(Policy or None): The worker's agent.
-        env(gym.Env or None): The worker's environment.
+        agent (Policy or None): The worker's agent.
+        env (Environment or None): The worker's environment.
 
     """
 
     def __init__(self,
                  *,
                  seed,
-                 max_path_length,
+                 max_episode_length,
                  worker_number,
                  deterministic=False,
                  accum_context=False):
         self._deterministic = deterministic
         self._accum_context = accum_context
+        self._episode_info = None
         super().__init__(seed=seed,
-                         max_path_length=max_path_length,
+                         max_episode_length=max_episode_length,
                          worker_number=worker_number)
 
-    def start_rollout(self):
-        """Begin a new rollout."""
-        self._path_length = 0
-        self._prev_obs = self.env.reset()
+    def start_episode(self):
+        """Begin a new episode."""
+        self._eps_length = 0
+        self._prev_obs, self._episode_info = self.env.reset()
 
-    def step_rollout(self):
-        """Take a single time-step in the current rollout.
+    def step_episode(self):
+        """Take a single time-step in the current episode.
 
         Returns:
-            bool: True iff the path is done, either due to the environment
-            indicating termination of due to reaching `max_path_length`.
+            bool: True iff the episode is done, either due to the environment
+            indicating termination of due to reaching `max_episode_length`.
 
         """
-        if self._path_length < self._max_path_length:
+        if self._eps_length < self._max_episode_length:
             a, agent_info = self.agent.get_action(self._prev_obs)
             if self._deterministic:
                 a = agent_info['mean']
-            next_o, r, d, env_info = self.env.step(a)
+            a, agent_info = self.agent.get_action(self._prev_obs)
+            es = self.env.step(a)
             self._observations.append(self._prev_obs)
-            self._rewards.append(r)
-            self._actions.append(a)
+            self._env_steps.append(es)
             for k, v in agent_info.items():
                 self._agent_infos[k].append(v)
-            for k, v in env_info.items():
-                self._env_infos[k].append(v)
-            self._path_length += 1
-            self._terminals.append(d)
+            self._eps_length += 1
+
             if self._accum_context:
-                s = TimeStep(env_spec=self.env,
-                             observation=self._prev_obs,
-                             next_observation=next_o,
-                             action=a,
-                             reward=float(r),
-                             terminal=d,
-                             env_info=env_info,
-                             agent_info=agent_info)
+                s = TimeStep.from_env_step(env_step=es,
+                                           last_observation=self._prev_obs,
+                                           agent_info=agent_info,
+                                           episode_info=self._episode_info)
                 self.agent.update_context(s)
-            if not d:
-                self._prev_obs = next_o
+            if not es.last:
+                self._prev_obs = es.observation
                 return False
-        self._lengths.append(self._path_length)
+        self._lengths.append(self._eps_length)
         self._last_observations.append(self._prev_obs)
         return True
 
     def rollout(self):
-        """Sample a single rollout of the agent in the environment.
+        """Sample a single episode of the agent in the environment.
 
         Returns:
-            garage.TrajectoryBatch: The collected trajectory.
+            EpisodeBatch: The collected episode.
 
         """
         self.agent.sample_from_belief()
-        self.start_rollout()
-        while not self.step_rollout():
+        self.start_episode()
+        while not self.step_episode():
             pass
-        self._agent_infos['context'] = [self.agent.z.detach().cpu().numpy()
-                                        ] * self._max_path_length
-        return self.collect_rollout()
+        return self.collect_episode()
