@@ -1,4 +1,5 @@
 """This modules creates a sac model in PyTorch."""
+# yapf: disable
 from collections import deque
 import copy
 
@@ -7,11 +8,11 @@ import numpy as np
 import torch
 import torch.nn.functional as F
 
-from garage import log_performance
-from garage.np import obtain_evaluation_samples
+from garage import log_performance, obtain_evaluation_episodes, StepType
 from garage.np.algos import RLAlgorithm
-from garage.sampler import OffPolicyVectorizedSampler
-from garage.torch import dict_np_to_torch, global_device
+from garage.torch import as_torch_dict, global_device
+
+# yapf: enable
 
 
 class SAC(RLAlgorithm):
@@ -39,16 +40,15 @@ class SAC(RLAlgorithm):
         qf2 (garage.torch.q_function.ContinuousMLPQFunction): QFunction/Critic
             used for actor/policy optimization. See Soft Actor-Critic and
             Applications.
-        replay_buffer (garage.replay_buffer.ReplayBuffer): Stores transitions
-            that are previously collected by the sampler.
-        env_spec (garage.envs.env_spec.EnvSpec): The env_spec attribute of the
-            environment that the agent is being trained in. Usually accessable
-            by calling env.spec.
-        max_path_length (int): Max path length of the algorithm.
-        max_eval_path_length (int or None): Maximum length of paths used for
-            off-policy evaluation. If None, defaults to `max_path_length`.
+        replay_buffer (ReplayBuffer): Stores transitions that are previously
+            collected by the sampler.
+        sampler (garage.sampler.Sampler): Sampler.
+        env_spec (EnvSpec): The env_spec attribute of the environment that the
+            agent is being trained in.
+        max_episode_length_eval (int or None): Maximum length of episodes used
+            for off-policy evaluation. If None, defaults to
+            `env_spec.max_episode_length`.
         gradient_steps_per_itr (int): Number of optimization steps that should
-        max_path_length(int): Max path length of the environment.
         gradient_steps_per_itr(int): Number of optimization steps that should
             occur before the training step is over and a new batch of
             transitions is collected by the sampler.
@@ -77,11 +77,12 @@ class SAC(RLAlgorithm):
             policy/actor, q_functions/critics, and temperature/entropy
             optimizations.
         steps_per_epoch (int): Number of train_once calls per epoch.
-        num_evaluation_trajectories (int): The number of evaluation
-            trajectories used for computing eval stats at the end of every
-            epoch.
-        eval_env (garage.envs.GarageEnv): environment used for collecting
-            evaluation trajectories. If None, a copy of the train env is used.
+        num_evaluation_episodes (int): The number of evaluation episodes used
+            for computing eval stats at the end of every epoch.
+        eval_env (Environment): environment used for collecting evaluation
+            episodes. If None, a copy of the train env is used.
+        use_deterministic_evaluation (bool): True if the trained policy
+            should be evaluated deterministically.
 
     """
 
@@ -92,9 +93,9 @@ class SAC(RLAlgorithm):
             qf1,
             qf2,
             replay_buffer,
+            sampler,
             *,  # Everything after this is numbers.
-            max_path_length,
-            max_eval_path_length=None,
+            max_episode_length_eval=None,
             gradient_steps_per_itr,
             fixed_alpha=None,
             target_entropy=None,
@@ -108,8 +109,9 @@ class SAC(RLAlgorithm):
             reward_scale=1.0,
             optimizer=torch.optim.Adam,
             steps_per_epoch=1,
-            num_evaluation_trajectories=10,
-            eval_env=None):
+            num_evaluation_episodes=10,
+            eval_env=None,
+            use_deterministic_evaluation=True):
 
         self._qf1 = qf1
         self._qf2 = qf2
@@ -120,7 +122,7 @@ class SAC(RLAlgorithm):
         self._initial_log_entropy = initial_log_entropy
         self._gradient_steps = gradient_steps_per_itr
         self._optimizer = optimizer
-        self._num_evaluation_trajectories = num_evaluation_trajectories
+        self._num_evaluation_episodes = num_evaluation_episodes
         self._eval_env = eval_env
 
         self._min_buffer_size = min_buffer_size
@@ -128,16 +130,18 @@ class SAC(RLAlgorithm):
         self._buffer_batch_size = buffer_batch_size
         self._discount = discount
         self._reward_scale = reward_scale
-        self.max_path_length = max_path_length
-        self._max_eval_path_length = max_eval_path_length
+        self.max_episode_length = env_spec.max_episode_length
+        self._max_episode_length_eval = env_spec.max_episode_length
+
+        if max_episode_length_eval is not None:
+            self._max_episode_length_eval = max_episode_length_eval
+        self._use_deterministic_evaluation = use_deterministic_evaluation
 
-        # used by OffPolicyVectorizedSampler
         self.policy = policy
         self.env_spec = env_spec
         self.replay_buffer = replay_buffer
-        self.exploration_policy = None
 
-        self.sampler_cls = OffPolicyVectorizedSampler
+        self._sampler = sampler
 
         self._reward_scale = reward_scale
         # use 2 target q networks
@@ -166,12 +170,12 @@ class SAC(RLAlgorithm):
             self._log_alpha = torch.Tensor([self._fixed_alpha]).log()
         self.episode_rewards = deque(maxlen=30)
 
-    def train(self, runner):
+    def train(self, trainer):
         """Obtain samplers and start actual training for each epoch.
 
         Args:
-            runner (LocalRunner): LocalRunner is passed to give algorithm
-                the access to runner.step_epochs(), which provides services
+            trainer (Trainer): Gives the algorithm the access to
+                :method:`~Trainer.step_epochs()`, which provides services
                 such as snapshotting and sampler control.
 
         Returns:
@@ -179,34 +183,37 @@ class SAC(RLAlgorithm):
 
         """
         if not self._eval_env:
-            self._eval_env = runner.get_env_copy()
+            self._eval_env = trainer.get_env_copy()
         last_return = None
-        for _ in runner.step_epochs():
+        for _ in trainer.step_epochs():
             for _ in range(self._steps_per_epoch):
                 if not (self.replay_buffer.n_transitions_stored >=
                         self._min_buffer_size):
                     batch_size = int(self._min_buffer_size)
                 else:
                     batch_size = None
-                runner.step_path = runner.obtain_samples(
-                    runner.step_itr, batch_size)
+                trainer.step_episode = trainer.obtain_samples(
+                    trainer.step_itr, batch_size)
                 path_returns = []
-                for path in runner.step_path:
+                for path in trainer.step_episode:
                     self.replay_buffer.add_path(
                         dict(observation=path['observations'],
                              action=path['actions'],
                              reward=path['rewards'].reshape(-1, 1),
                              next_observation=path['next_observations'],
-                             terminal=path['dones'].reshape(-1, 1)))
+                             terminal=np.array([
+                                 step_type == StepType.TERMINAL
+                                 for step_type in path['step_types']
+                             ]).reshape(-1, 1)))
                     path_returns.append(sum(path['rewards']))
-                assert len(path_returns) is len(runner.step_path)
+                assert len(path_returns) == len(trainer.step_episode)
                 self.episode_rewards.append(np.mean(path_returns))
                 for _ in range(self._gradient_steps):
                     policy_loss, qf1_loss, qf2_loss = self.train_once()
-            last_return = self._evaluate_policy(runner.step_itr)
+            last_return = self._evaluate_policy(trainer.step_itr)
             self._log_statistics(policy_loss, qf1_loss, qf2_loss)
-            tabular.record('TotalEnvSteps', runner.total_env_steps)
-            runner.step_itr += 1
+            tabular.record('TotalEnvSteps', trainer.total_env_steps)
+            trainer.step_itr += 1
 
         return np.mean(last_return)
 
@@ -229,7 +236,7 @@ class SAC(RLAlgorithm):
         if self.replay_buffer.n_transitions_stored >= self._min_buffer_size:
             samples = self.replay_buffer.sample_transitions(
                 self._buffer_batch_size)
-            samples = dict_np_to_torch(samples)
+            samples = as_torch_dict(samples)
             policy_loss, qf1_loss, qf2_loss = self.optimize_policy(samples)
             self._update_targets()
 
@@ -299,10 +306,10 @@ class SAC(RLAlgorithm):
             samples_data (dict): Transitions(S,A,R,S') that are sampled from
                 the replay buffer. It should have the keys 'observation',
                 'action', 'reward', 'terminal', and 'next_observations'.
-            new_actions(torch.Tensor): Actions resampled from the policy based
+            new_actions (torch.Tensor): Actions resampled from the policy based
                 based on the Observations, obs, which were sampled from the
                 replay buffer. Shape is (action_dim, buffer_batch_size).
-            log_pi_new_actions(torch.Tensor): Log probability of the new
+            log_pi_new_actions (torch.Tensor): Log probability of the new
                 actions on the TanhNormal distributions that they were sampled
                 from. Shape is (1, buffer_batch_size).
 
@@ -445,25 +452,27 @@ class SAC(RLAlgorithm):
         return policy_loss, qf1_loss, qf2_loss
 
     def _evaluate_policy(self, epoch):
-        """Evaluate the performance of the policy via deterministic rollouts.
+        """Evaluate the performance of the policy via deterministic sampling.
 
             Statistics such as (average) discounted return and success rate are
             recorded.
 
         Args:
-            epoch(int): The current training epoch.
+            epoch (int): The current training epoch.
 
         Returns:
-            float: The average return across self._num_evaluation_trajectories
-                trajectories
+            float: The average return across self._num_evaluation_episodes
+                episodes
 
         """
-        eval_trajectories = obtain_evaluation_samples(
+        eval_episodes = obtain_evaluation_episodes(
             self.policy,
             self._eval_env,
-            num_trajs=self._num_evaluation_trajectories)
+            self._max_episode_length_eval,
+            num_eps=self._num_evaluation_episodes,
+            deterministic=self._use_deterministic_evaluation)
         last_return = log_performance(epoch,
-                                      eval_trajectories,
+                                      eval_episodes,
                                       discount=self._discount)
         return last_return
 
@@ -471,9 +480,9 @@ class SAC(RLAlgorithm):
         """Record training statistics to dowel such as losses and returns.
 
         Args:
-            policy_loss(torch.Tensor): loss from actor/policy network.
-            qf1_loss(torch.Tensor): loss from 1st qf/critic network.
-            qf2_loss(torch.Tensor): loss from 2nd qf/critic network.
+            policy_loss (torch.Tensor): loss from actor/policy network.
+            qf1_loss (torch.Tensor): loss from 1st qf/critic network.
+            qf2_loss (torch.Tensor): loss from 2nd qf/critic network.
 
         """
         with torch.no_grad():
