@@ -1,9 +1,9 @@
+from garage.envs import GymEnv
 from garage.np.algos import CMAES
-from garage.np.baselines import LinearFeatureBaseline
-from garage.sampler import OnPolicyVectorizedSampler
-from garage.tf.envs import TfEnv
-from garage.tf.experiment import LocalTFRunner
+from garage.sampler import LocalSampler
 from garage.tf.policies import CategoricalMLPPolicy
+from garage.trainer import TFTrainer
+
 from tests.fixtures import snapshot_config, TfGraphTestCase
 
 
@@ -11,25 +11,28 @@ class TestCMAES(TfGraphTestCase):
 
     def test_cma_es_cartpole(self):
         """Test CMAES with Cartpole-v1 environment."""
-        with LocalTFRunner(snapshot_config) as runner:
-            env = TfEnv(env_name='CartPole-v1')
+        with TFTrainer(snapshot_config) as trainer:
+            env = GymEnv('CartPole-v1')
 
             policy = CategoricalMLPPolicy(name='policy',
                                           env_spec=env.spec,
                                           hidden_sizes=(32, 32))
 
-            baseline = LinearFeatureBaseline(env_spec=env.spec)
-
             n_samples = 20
 
+            sampler = LocalSampler(
+                agents=policy,
+                envs=env,
+                max_episode_length=env.spec.max_episode_length,
+                is_tf_worker=True)
+
             algo = CMAES(env_spec=env.spec,
                          policy=policy,
-                         baseline=baseline,
-                         max_path_length=100,
+                         sampler=sampler,
                          n_samples=n_samples)
 
-            runner.setup(algo, env, sampler_cls=OnPolicyVectorizedSampler)
-            runner.train(n_epochs=1, batch_size=1000, n_epoch_cycles=n_samples)
+            trainer.setup(algo, env)
+            trainer.train(n_epochs=1, batch_size=1000)
             # No assertion on return because CMAES is not stable.
 
             env.close()
