@@ -4,18 +4,20 @@ too low.
 """
 import pickle
 
-import gym
 import numpy as np
 import pytest
 import tensorflow as tf
 
-from garage.envs import GarageEnv
-from garage.experiment import LocalTFRunner
+from garage.envs import GymEnv
+from garage.experiment import deterministic
 from garage.np.exploration_policies import EpsilonGreedyPolicy
 from garage.replay_buffer import PathBuffer
+from garage.sampler import FragmentWorker, LocalSampler
 from garage.tf.algos import DQN
-from garage.tf.policies import DiscreteQfDerivedPolicy
+from garage.tf.policies import DiscreteQFArgmaxPolicy
 from garage.tf.q_functions import DiscreteMLPQFunction
+from garage.trainer import TFTrainer
+
 from tests.fixtures import snapshot_config, TfGraphTestCase
 
 
@@ -24,15 +26,16 @@ class TestDQN(TfGraphTestCase):
     @pytest.mark.large
     def test_dqn_cartpole(self):
         """Test DQN with CartPole environment."""
-        with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
+        deterministic.set_seed(100)
+        with TFTrainer(snapshot_config, sess=self.sess) as trainer:
             n_epochs = 10
             steps_per_epoch = 10
             sampler_batch_size = 500
             num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
-            env = GarageEnv(gym.make('CartPole-v0'))
+            env = GymEnv('CartPole-v0')
             replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
-            policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
+            policy = DiscreteQFArgmaxPolicy(env_spec=env.spec, qf=qf)
             epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
                 policy=policy,
@@ -40,11 +43,18 @@ class TestDQN(TfGraphTestCase):
                 max_epsilon=1.0,
                 min_epsilon=0.02,
                 decay_ratio=0.1)
+            sampler = LocalSampler(
+                agents=epilson_greedy_policy,
+                envs=env,
+                max_episode_length=env.spec.max_episode_length,
+                is_tf_worker=True,
+                worker_class=FragmentWorker)
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
                        exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
+                       sampler=sampler,
                        qf_lr=1e-4,
                        discount=1.0,
                        min_buffer_size=int(1e3),
@@ -54,25 +64,26 @@ class TestDQN(TfGraphTestCase):
                        target_network_update_freq=1,
                        buffer_batch_size=32)
 
-            runner.setup(algo, env)
-            last_avg_ret = runner.train(n_epochs=n_epochs,
-                                        batch_size=sampler_batch_size)
-            assert last_avg_ret > 15
+            trainer.setup(algo, env)
+            last_avg_ret = trainer.train(n_epochs=n_epochs,
+                                         batch_size=sampler_batch_size)
+            assert last_avg_ret > 8.8
 
             env.close()
 
     @pytest.mark.large
     def test_dqn_cartpole_double_q(self):
         """Test DQN with CartPole environment."""
-        with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
+        deterministic.set_seed(100)
+        with TFTrainer(snapshot_config, sess=self.sess) as trainer:
             n_epochs = 10
             steps_per_epoch = 10
             sampler_batch_size = 500
             num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
-            env = GarageEnv(gym.make('CartPole-v0'))
+            env = GymEnv('CartPole-v0')
             replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
-            policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
+            policy = DiscreteQFArgmaxPolicy(env_spec=env.spec, qf=qf)
             epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
                 policy=policy,
@@ -80,11 +91,18 @@ class TestDQN(TfGraphTestCase):
                 max_epsilon=1.0,
                 min_epsilon=0.02,
                 decay_ratio=0.1)
+            sampler = LocalSampler(
+                agents=epilson_greedy_policy,
+                envs=env,
+                max_episode_length=env.spec.max_episode_length,
+                is_tf_worker=True,
+                worker_class=FragmentWorker)
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
                        exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
+                       sampler=sampler,
                        qf_lr=1e-4,
                        discount=1.0,
                        min_buffer_size=int(1e3),
@@ -94,25 +112,26 @@ class TestDQN(TfGraphTestCase):
                        target_network_update_freq=1,
                        buffer_batch_size=32)
 
-            runner.setup(algo, env)
-            last_avg_ret = runner.train(n_epochs=n_epochs,
-                                        batch_size=sampler_batch_size)
-            assert last_avg_ret > 15
+            trainer.setup(algo, env)
+            last_avg_ret = trainer.train(n_epochs=n_epochs,
+                                         batch_size=sampler_batch_size)
+            assert last_avg_ret > 8.8
 
             env.close()
 
     @pytest.mark.large
     def test_dqn_cartpole_grad_clip(self):
         """Test DQN with CartPole environment."""
-        with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
+        deterministic.set_seed(100)
+        with TFTrainer(snapshot_config, sess=self.sess) as trainer:
             n_epochs = 10
             steps_per_epoch = 10
             sampler_batch_size = 500
             num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
-            env = GarageEnv(gym.make('CartPole-v0'))
+            env = GymEnv('CartPole-v0')
             replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
-            policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
+            policy = DiscreteQFArgmaxPolicy(env_spec=env.spec, qf=qf)
             epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
                 policy=policy,
@@ -120,11 +139,18 @@ class TestDQN(TfGraphTestCase):
                 max_epsilon=1.0,
                 min_epsilon=0.02,
                 decay_ratio=0.1)
+            sampler = LocalSampler(
+                agents=epilson_greedy_policy,
+                envs=env,
+                max_episode_length=env.spec.max_episode_length,
+                is_tf_worker=True,
+                worker_class=FragmentWorker)
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
                        exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
+                       sampler=sampler,
                        qf_lr=1e-4,
                        discount=1.0,
                        min_buffer_size=int(1e3),
@@ -135,24 +161,25 @@ class TestDQN(TfGraphTestCase):
                        target_network_update_freq=1,
                        buffer_batch_size=32)
 
-            runner.setup(algo, env)
-            last_avg_ret = runner.train(n_epochs=n_epochs,
-                                        batch_size=sampler_batch_size)
-            assert last_avg_ret > 15
+            trainer.setup(algo, env)
+            last_avg_ret = trainer.train(n_epochs=n_epochs,
+                                         batch_size=sampler_batch_size)
+            assert last_avg_ret > 8.8
 
             env.close()
 
     def test_dqn_cartpole_pickle(self):
         """Test DQN with CartPole environment."""
-        with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
+        deterministic.set_seed(100)
+        with TFTrainer(snapshot_config, sess=self.sess) as trainer:
             n_epochs = 10
             steps_per_epoch = 10
             sampler_batch_size = 500
             num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
-            env = GarageEnv(gym.make('CartPole-v0'))
+            env = GymEnv('CartPole-v0')
             replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
-            policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
+            policy = DiscreteQFArgmaxPolicy(env_spec=env.spec, qf=qf)
             epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
                 policy=policy,
@@ -160,11 +187,18 @@ class TestDQN(TfGraphTestCase):
                 max_epsilon=1.0,
                 min_epsilon=0.02,
                 decay_ratio=0.1)
+            sampler = LocalSampler(
+                agents=epilson_greedy_policy,
+                envs=env,
+                max_episode_length=env.spec.max_episode_length,
+                is_tf_worker=True,
+                worker_class=FragmentWorker)
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
                        exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
+                       sampler=sampler,
                        qf_lr=1e-4,
                        discount=1.0,
                        min_buffer_size=int(1e3),
@@ -174,9 +208,9 @@ class TestDQN(TfGraphTestCase):
                        steps_per_epoch=steps_per_epoch,
                        target_network_update_freq=1,
                        buffer_batch_size=32)
-            runner.setup(algo, env)
+            trainer.setup(algo, env)
             with tf.compat.v1.variable_scope(
-                    'DiscreteMLPQFunction/MLPModel/mlp/hidden_0', reuse=True):
+                    'DiscreteMLPQFunction/mlp/hidden_0', reuse=True):
                 bias = tf.compat.v1.get_variable('bias')
                 # assign it to all one
                 old_bias = tf.ones_like(bias).eval()
@@ -186,8 +220,7 @@ class TestDQN(TfGraphTestCase):
             with tf.compat.v1.Session(graph=tf.Graph()):
                 pickle.loads(h)
                 with tf.compat.v1.variable_scope(
-                        'DiscreteMLPQFunction/MLPModel/mlp/hidden_0',
-                        reuse=True):
+                        'DiscreteMLPQFunction/mlp/hidden_0', reuse=True):
                     new_bias = tf.compat.v1.get_variable('bias')
                     new_bias = new_bias.eval()
                     assert np.array_equal(old_bias, new_bias)
