@@ -4,9 +4,9 @@ garage.tf.baselines failed to initialize.
 """
 import tensorflow as tf
 
-from garage.tf.baselines import ContinuousMLPBaseline
-from garage.tf.baselines import GaussianMLPBaseline
-from garage.tf.envs import TfEnv
+from garage.envs import GymEnv
+from garage.tf.baselines import ContinuousMLPBaseline, GaussianMLPBaseline
+
 from tests.fixtures import TfGraphTestCase
 from tests.fixtures.envs.dummy import DummyBoxEnv
 
@@ -15,12 +15,12 @@ class TestTfBaselines(TfGraphTestCase):
 
     def test_baseline(self):
         """Test the baseline initialization."""
-        box_env = TfEnv(DummyBoxEnv())
+        box_env = GymEnv(DummyBoxEnv())
         deterministic_mlp_baseline = ContinuousMLPBaseline(env_spec=box_env)
         gaussian_mlp_baseline = GaussianMLPBaseline(env_spec=box_env)
 
         self.sess.run(tf.compat.v1.global_variables_initializer())
-        deterministic_mlp_baseline.get_param_values(trainable=True)
-        gaussian_mlp_baseline.get_param_values(trainable=True)
+        deterministic_mlp_baseline.get_param_values()
+        gaussian_mlp_baseline.get_param_values()
 
         box_env.close()
