@@ -1,20 +1,21 @@
 """Module for testing SAC loss functions."""
 from unittest.mock import MagicMock
 
-import gym
 import numpy as np
 import pytest
 import torch
 from torch.nn import functional as F
 
-from garage.envs import GarageEnv, normalize
-from garage.experiment import deterministic, LocalRunner
+from garage.envs import GymEnv, normalize
+from garage.experiment import deterministic
 from garage.replay_buffer import PathBuffer
-from garage.sampler import LocalSampler
+from garage.sampler import FragmentWorker, LocalSampler
 from garage.torch import set_gpu_mode
 from garage.torch.algos import SAC
 from garage.torch.policies import TanhGaussianMLPPolicy
 from garage.torch.q_functions import ContinuousMLPQFunction
+from garage.trainer import Trainer
+
 from tests.fixtures import snapshot_config
 
 
@@ -83,16 +84,18 @@ def testCriticLoss():
     """Test Sac Critic/QF loss."""
     # pylint: disable=no-member
     policy = DummyActorPolicy()
-    sac = SAC(env_spec=None,
+    spec = MagicMock
+    spec.max_episode_length = 100
+    sac = SAC(env_spec=spec,
               policy=policy,
               qf1=DummyCriticNet(),
               qf2=DummyCriticNet(),
+              sampler=None,
               replay_buffer=None,
               gradient_steps_per_itr=1,
               discount=0.9,
               buffer_batch_size=2,
               target_entropy=3.0,
-              max_path_length=10,
               optimizer=MagicMock)
 
     observations = torch.FloatTensor([[1, 2], [3, 4]])
@@ -121,17 +124,19 @@ def testActorLoss():
     """Test Sac Actor/Policy loss."""
     # pylint: disable=no-member
     policy = DummyActorPolicy()
-    sac = SAC(env_spec=None,
+    policy = DummyActorPolicy()
+    spec = MagicMock
+    sac = SAC(env_spec=spec,
               policy=policy,
               qf1=DummyCriticNet(),
               qf2=DummyCriticNet(),
+              sampler=None,
               replay_buffer=None,
               discount=1,
               buffer_batch_size=2,
               target_entropy=3.0,
               initial_log_entropy=0,
               optimizer=MagicMock,
-              max_path_length=10,
               gradient_steps_per_itr=1)
 
     observations = torch.Tensor([[1., 2.], [3., 4.]])
@@ -148,17 +153,19 @@ def testTemperatureLoss():
     """Test Sac temperature loss."""
     # pylint: disable=no-member
     policy = DummyActorPolicy()
-    sac = SAC(env_spec=None,
+    policy = DummyActorPolicy()
+    spec = MagicMock
+    sac = SAC(env_spec=spec,
               policy=policy,
               qf1=DummyCriticNet(),
               qf2=DummyCriticNet(),
+              sampler=None,
               replay_buffer=None,
               discount=1,
               buffer_batch_size=2,
               target_entropy=3.0,
               initial_log_entropy=4.0,
               optimizer=MagicMock,
-              max_path_length=10,
               gradient_steps_per_itr=1)
     observations = torch.Tensor([[1., 2.], [3., 4.]])
     action_dists = policy(observations)[0]
@@ -174,7 +181,8 @@ def testTemperatureLoss():
 def test_sac_inverted_double_pendulum():
     """Test Sac performance on inverted pendulum."""
     # pylint: disable=unexpected-keyword-arg
-    env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+    env = normalize(GymEnv('InvertedDoublePendulum-v2',
+                           max_episode_length=100))
     deterministic.set_seed(0)
     policy = TanhGaussianMLPPolicy(
         env_spec=env.spec,
@@ -193,13 +201,17 @@ def test_sac_inverted_double_pendulum():
                                  hidden_sizes=[32, 32],
                                  hidden_nonlinearity=F.relu)
     replay_buffer = PathBuffer(capacity_in_transitions=int(1e6), )
-    runner = LocalRunner(snapshot_config=snapshot_config)
+    trainer = Trainer(snapshot_config=snapshot_config)
+    sampler = LocalSampler(agents=policy,
+                           envs=env,
+                           max_episode_length=env.spec.max_episode_length,
+                           worker_class=FragmentWorker)
     sac = SAC(env_spec=env.spec,
               policy=policy,
               qf1=qf1,
               qf2=qf2,
+              sampler=sampler,
               gradient_steps_per_itr=100,
-              max_path_length=100,
               replay_buffer=replay_buffer,
               min_buffer_size=1e3,
               target_update_tau=5e-3,
@@ -207,13 +219,13 @@ def test_sac_inverted_double_pendulum():
               buffer_batch_size=64,
               reward_scale=1.,
               steps_per_epoch=2)
-    runner.setup(sac, env, sampler_cls=LocalSampler)
+    trainer.setup(sac, env)
     if torch.cuda.is_available():
         set_gpu_mode(True)
     else:
         set_gpu_mode(False)
     sac.to()
-    ret = runner.train(n_epochs=12, batch_size=200, plot=False)
+    ret = trainer.train(n_epochs=12, batch_size=200, plot=False)
     # check that automatic entropy tuning is used
     assert sac._use_automatic_entropy_tuning
     # assert that there was a gradient properly connected to alpha
@@ -221,14 +233,15 @@ def test_sac_inverted_double_pendulum():
     # correct.
     assert not torch.allclose(torch.Tensor([1.]), sac._log_alpha.to('cpu'))
     # check that policy is learning beyond predecided threshold
-    assert ret > 85
+    assert ret > 80
 
 
 @pytest.mark.mujoco
 def test_fixed_alpha():
     """Test if using fixed_alpha ensures that alpha is non differentiable."""
     # pylint: disable=unexpected-keyword-arg
-    env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+    env = normalize(GymEnv('InvertedDoublePendulum-v2',
+                           max_episode_length=100))
     deterministic.set_seed(0)
     policy = TanhGaussianMLPPolicy(
         env_spec=env.spec,
@@ -247,13 +260,17 @@ def test_fixed_alpha():
                                  hidden_sizes=[32, 32],
                                  hidden_nonlinearity=F.relu)
     replay_buffer = PathBuffer(capacity_in_transitions=int(1e6), )
-    runner = LocalRunner(snapshot_config=snapshot_config)
+    trainer = Trainer(snapshot_config=snapshot_config)
+    sampler = LocalSampler(agents=policy,
+                           envs=env,
+                           max_episode_length=env.spec.max_episode_length,
+                           worker_class=FragmentWorker)
     sac = SAC(env_spec=env.spec,
               policy=policy,
               qf1=qf1,
               qf2=qf2,
+              sampler=sampler,
               gradient_steps_per_itr=100,
-              max_path_length=100,
               replay_buffer=replay_buffer,
               min_buffer_size=100,
               target_update_tau=5e-3,
@@ -262,8 +279,8 @@ def test_fixed_alpha():
               reward_scale=1.,
               steps_per_epoch=1,
               fixed_alpha=np.exp(0.5))
-    runner.setup(sac, env, sampler_cls=LocalSampler)
+    trainer.setup(sac, env)
     sac.to()
-    runner.train(n_epochs=1, batch_size=100, plot=False)
+    trainer.train(n_epochs=1, batch_size=100, plot=False)
     assert torch.allclose(torch.Tensor([0.5]), sac._log_alpha.cpu())
     assert not sac._use_automatic_entropy_tuning
